package com.caimei365.wechat.controller;

import com.alibaba.fastjson.JSONObject;
import com.caimei365.wechat.dao.WeChatDao;
import com.caimei365.wechat.entity.WeChatUser;
import com.caimei365.wechat.utils.HttpUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.List;

/**
 * 微信接口调试API
 *
 * @author : Charles
 * @date : 2022/1/13
 */
@Slf4j
@RestController
public class WechatDebugApi {

    @Value("${wechat.apiUrl}")
    private String wechatApiUrl;
    @Value("${wechat.caimei.appid}")
    private String caimeiAppid;
    @Value("${wechat.caimei.secret}")
    private String caimeiSecret;
    @Resource
    private WeChatDao weChatDao;
    /**
     * 获取微信 access_token
     */
    @GetMapping("/wechat/debug/access_token")
    public String getAccessToken(String appid, String secret) {
        String url = wechatApiUrl + "/cgi-bin/token?grant_type=client_credential&appid=" + appid + "&secret=" + secret;
        String result = HttpUtil.httpRequest(url, "GET", null);
        log.info("【微信接口调试】获取access_token：{}", result);
        return result;
    }

    /**
     * 获取微信用户列表
     * @param openid 初始openid，不填默认从头开始拉取
     */
    @GetMapping("/wechat/debug/user/list")
    public String getUserList(String token, String  openid) {
        String url =wechatApiUrl + "/cgi-bin/user/get?access_token="+ token + "&next_openid="+ openid;
        String result = HttpUtil.httpRequest(url, "GET", null);
        log.info("【微信接口调试】获取用户列表：{}", result);
        return result;
    }

    /**
     * 公众号迁移后更新OpenId
     * 帐号迁移15天后，该转换接口将会失效、无法拉取到数据
     * https://api.weixin.qq.com/cgi-bin/changeopenid?access_token=
     */
    @PostMapping("/wechat/debug/change/openid")
    public String changeOpenId(@RequestBody String json, String token) {
        String url = wechatApiUrl + "/cgi-bin/changeopenid?access_token="+ token;
        String result = HttpUtil.httpRequest(url, "POST", json);
        log.info("【微信接口调试】公众号迁移后更新OpenId：{}", result);
        return result;
    }

    /**
     * 关注用户更新unionId
     */
    @PostMapping("/wechat/debug/user/union")
    public String changeOpenId(String token) {
        List<WeChatUser> userList = weChatDao.getWeChatUserList();
        for (WeChatUser user : userList) {
            if(!StringUtils.hasLength(user.getUnionId())) {
                String openid = user.getOpenid();
                try {
                    // 获取用户unionId：/cgi-bin/user/info?access_token=ACCESS_TOKEN&openid=OPENID&lang=zh_CN
                    String unionIdUrl = wechatApiUrl + "/cgi-bin/user/info?access_token="+ token + "&openid=" + openid + "&lang=zh_CN";
                    String result2 = HttpUtil.httpRequest(unionIdUrl, "GET", null);
                    log.info("openid:" + openid + "，获取用户unionId：" + result2);
                    JSONObject parse2 = JSONObject.parseObject(result2, JSONObject.class);
                    String unionId = parse2.getString("unionid");
                    user.setUnionId(unionId);
                    weChatDao.updateWeChatUser(user);
                } catch (Exception e) {
                    log.error("获取unionId失败:", e);
                }
            }
        }
        return "【微信接口调试】关注用户更新unionId";
    }
}
